# -*- coding: utf-8 -*-
import os
import copy
import pytest

from boussole.conf.patcher import SettingsPatcher


def test_conf_patcher_001_expand_nothingtodo(settings, 
                                             sample_project_settings):
    """conf.patcher.SettingsPatcher: Nothing to expand, path is correct"""
    patcher = SettingsPatcher()
    
    expand_result = patcher._patch_expand_path(sample_project_settings, "DUMMY_NAME", "/foo/bar")
    
    assert expand_result == "/foo/bar"


def test_conf_patcher_002_expand_homedir(settings, 
                                             sample_project_settings):
    """conf.patcher.SettingsPatcher: Single path expand home user dir"""
    patcher = SettingsPatcher()
    
    expand_result = patcher._patch_expand_path(sample_project_settings, "DUMMY_NAME", "~/foo")
    
    assert expand_result == os.path.join(os.environ['HOME'], 'foo')


def test_conf_patcher_003_expand_absolute(settings, 
                                             sample_project_settings):
    """conf.patcher.SettingsPatcher: Single path expand to absolute dir"""
    patcher = SettingsPatcher()
    patcher.basedir = settings.fixtures_path
    
    expand_result = patcher._patch_expand_path(sample_project_settings,
                                               "DUMMY_NAME",
                                               "tests/data_fixtures/sample_project")
    
    assert expand_result == os.path.join(settings.fixtures_path,
                                         "tests/data_fixtures/sample_project")


def test_conf_patcher_004_expand_wrong(settings, 
                                             sample_project_settings):
    """conf.patcher.SettingsPatcher: The path does not exists but is expanded
       to absolute from current dir"""
    patcher = SettingsPatcher()
    patcher.basedir = "/home/user"
    
    expand_result = patcher._patch_expand_path(sample_project_settings,
                                               "DUMMY_NAME", "foo/coco/bar")
    
    assert expand_result == os.path.join("/home/user", "foo/coco/bar")


def test_conf_patcher_005_expand_normpath(settings, 
                                             sample_project_settings):
    """conf.patcher.SettingsPatcher: Single path expand to absolute dir
       normalized"""
    patcher = SettingsPatcher()
    patcher.basedir = settings.fixtures_path
    
    expand_result = patcher._patch_expand_path(sample_project_settings,
                                               "DUMMY_NAME",
                                               "tests/data_fixtures/foo/../sample_project")
    
    assert expand_result == os.path.join(settings.fixtures_path,
                                         "tests/data_fixtures/sample_project")


def test_conf_patcher_010_expands(settings, sample_project_settings):
    """conf.patcher.SettingsPatcher: Expand a list of path"""
    patcher = SettingsPatcher()
    patcher.basedir = settings.fixtures_path
    
    current_dir = os.getcwd()
    
    paths = [
        "/foo/bar",
        "~/foo",
        "tests/data_fixtures/sample_project",
        "tests/data_fixtures/foo/../sample_project",
        "foo/coco/bar",
    ]
    
    expected = [
        "/foo/bar",
        os.path.join(os.environ['HOME'], 'foo'),
        os.path.join(settings.fixtures_path, "tests/data_fixtures/sample_project"),
        os.path.join(settings.fixtures_path, "tests/data_fixtures/sample_project"),
        os.path.join(settings.fixtures_path, "foo/coco/bar"),
    ]
    
    expand_result = patcher._patch_expand_paths(sample_project_settings, "DUMMY_NAME", paths)
    
    assert expand_result == expected



# -*- coding: utf-8 -*-
import os
import pytest

from boussole.conf import DEFAULT_SETTINGS
from boussole.conf.model import Settings


def test_conf_settings_init_001_default(settings, sample_project_settings):
    """conf.Settings: Create a empty Settings object width default values"""
    settings_object = Settings()

    assert settings_object._settings == DEFAULT_SETTINGS

    assert settings_object.TARGET_PATH == DEFAULT_SETTINGS['TARGET_PATH']
    assert settings_object.COMPILER_ARGS == DEFAULT_SETTINGS['COMPILER_ARGS']



"""
Main entrance to commandline actions
"""
import click

from boussole.cli.version import version_command
from boussole.cli.compile import compile_command
from boussole.cli.watch import watch_command


def main():
    print "Hello World!"


@click.group()
@click.option('--config', default=None, metavar='PATH',
              help='Path to a Boussole config file')
@click.pass_context
def cli_frontend(ctx, config):
    """
    Boussole is a commandline interface to build SASS projects using libsass.
    """
    # Init the default context that will be passed to commands
    # Not really used since settings is imported from its module
    ctx.obj = {}


# Attach commands methods to the main grouper
cli_frontend.add_command(version_command, name="version")
cli_frontend.add_command(compile_command, name="compile")
cli_frontend.add_command(watch_command, name="watch")



# -*- coding: utf-8 -*-
import os
import pytest

from boussole.exceptions import SettingsBackendError
from boussole.conf.json_backend import SettingsBackendJson


def test_settings_base_parse_ok_001(settings, sample_project_settings):
    """conf.json_backend.SettingsBackendJson: JSON content parsing"""
    settings_loader = SettingsBackendJson()

    filepath = settings_loader.get_filepath(settings.fixtures_path)

    content = settings_loader.open(filepath)

    assert settings_loader.parse(filepath, content) == sample_project_settings


def test_settings_base_parse_error_001(settings, sample_project_settings):
    """conf.json_backend.SettingsBackendJson: JSON content parsing error"""
    settings_loader = SettingsBackendJson()

    filepath = settings_loader.get_filepath(settings.fixtures_path, "settings.txt")

    content = settings_loader.open(filepath)

    with pytest.raises(SettingsBackendError):
        settings_loader.parse(filepath, content)



# -*- coding: utf-8 -*-
"""
JSON settings backend
=====================
"""
import json

from boussole.exceptions import SettingsBackendError
from boussole.conf.base_backend import SettingsBackendBase


class SettingsBackendJson(SettingsBackendBase):
    """
    JSON backend for settings
    """
    _default_filename = 'settings.json'

    def parse(self, filepath, content):
        """
        Parse opened settings content using JSON parser.

        Args:
            filepath (str): Settings object, depends from backend
            content (str): Settings content from opened file, depends from
                backend.

        Raises:
            boussole.exceptions.SettingsBackendError: If parser can not decode
                a valid JSON object.

        Returns:
            dict: Dictionnary containing parsed setting elements.

        """
        try:
            parsed = json.loads(content)
        except ValueError:
            msg = "No JSON object could be decoded from file: {}"
            raise SettingsBackendError(msg.format(filepath))
        return parsed



# -*- coding: utf-8 -*-
"""
Finder
======
"""
import os


class ScssFinder(object):
    """
    Project finder for SCSS sources

    Attributes:
        FINDER_COMPILABLE_EXTS: List of file extensions regarded as compilable
            sources.
    """
    FINDER_COMPILABLE_EXTS = ['scss', 'sass']

    def get_compilable_sources(self, paths, recursive=True):
        """
        Find all scss sources that should be compiled, aka all sources that
        are not "partials" SASS sources.

        This is a common method used by ``children`` and ``parents`` methods.

        Args:
            paths (list): Directory paths to scan.

        Keyword Arguments:
            recursive (bool): Switch to enabled recursive finding (if True).
                Default to True.

        Returns:
            set: List of source paths.

        """
        filepaths = []

        for sourcedir in paths:
            for root, dirs, files in os.walk(sourcedir):
                for item in files:
                    filename, ext = os.path.splitext(item)
                    ext = ext[1:]
                    if not filename.startswith('_') and \
                       ext in self.FINDER_COMPILABLE_EXTS:
                            filepaths.append(os.path.join(root, item))
                # For non recursive usage, break from the first entry
                if not recursive:
                    break

        return filepaths



# -*- coding: utf-8 -*-
import os
import pytest

def test_finder_compilable_001(settings, finder):
    """finder.ScssInspector: Find non recursively all compilable sources
       from sample project"""
    assert finder.get_compilable_sources([settings.sample_path], recursive=False) == [
        os.path.join(settings.sample_path, 'main_syntax.scss'),
        os.path.join(settings.sample_path, 'main_error.scss'),
        os.path.join(settings.sample_path, 'main_circular_5.scss'),
        os.path.join(settings.sample_path, 'main_depth_import-2.scss'),
        os.path.join(settings.sample_path, 'main_twins_2.scss'),
        os.path.join(settings.sample_path, 'main_twins_1.scss'),
        os.path.join(settings.sample_path, 'main_circular_0.scss'),
        os.path.join(settings.sample_path, 'main_circular_4.scss'),
        os.path.join(settings.sample_path, 'main_circular_3.scss'),
        os.path.join(settings.sample_path, 'main_twins_3.scss'),
        os.path.join(settings.sample_path, 'main_using_libs.scss'),
        os.path.join(settings.sample_path, 'main_depth_import-3.scss'),
        os.path.join(settings.sample_path, 'main_circular_1.scss'),
        os.path.join(settings.sample_path, 'main_depth_import-1.scss'),
        os.path.join(settings.sample_path, 'main_commented.scss'),
        os.path.join(settings.sample_path, 'main_with_subimports.scss'),
        os.path.join(settings.sample_path, 'main_circular_2.scss'),
        os.path.join(settings.sample_path, 'main_circular_bridge.scss'),
        os.path.join(settings.sample_path, 'main_basic.scss'),
    ]

def test_finder_compilable_002(settings, finder):
    """finder.ScssInspector: Find recursively all compilable sources from
       sample project"""
    assert finder.get_compilable_sources([settings.sample_path]) == [
        os.path.join(settings.sample_path, 'main_syntax.scss'),
        os.path.join(settings.sample_path, 'main_error.scss'),
        os.path.join(settings.sample_path, 'main_circular_5.scss'),
        os.path.join(settings.sample_path, 'main_depth_import-2.scss'),
        os.path.join(settings.sample_path, 'main_twins_2.scss'),
        os.path.join(settings.sample_path, 'main_twins_1.scss'),
        os.path.join(settings.sample_path, 'main_circular_0.scss'),
        os.path.join(settings.sample_path, 'main_circular_4.scss'),
        os.path.join(settings.sample_path, 'main_circular_3.scss'),
        os.path.join(settings.sample_path, 'main_twins_3.scss'),
        os.path.join(settings.sample_path, 'main_using_libs.scss'),
        os.path.join(settings.sample_path, 'main_depth_import-3.scss'),
        os.path.join(settings.sample_path, 'main_circular_1.scss'),
        os.path.join(settings.sample_path, 'main_depth_import-1.scss'),
        os.path.join(settings.sample_path, 'main_commented.scss'),
        os.path.join(settings.sample_path, 'main_with_subimports.scss'),
        os.path.join(settings.sample_path, 'main_circular_2.scss'),
        os.path.join(settings.sample_path, 'main_circular_bridge.scss'),
        os.path.join(settings.sample_path, 'main_basic.scss'),
        os.path.join(settings.sample_path, 'components/twin_3.scss'),
        os.path.join(settings.sample_path, 'components/twin_2.scss'),
    ]

def test_finder_compilable_003(settings, finder):
    """finder.ScssInspector: Find recursively all compilable sources from
       sample project subdirectory"""
    assert finder.get_compilable_sources([os.path.join(settings.sample_path, 'components')]) == [
        os.path.join(settings.sample_path, 'components/twin_3.scss'),
        os.path.join(settings.sample_path, 'components/twin_2.scss'),
    ]



# -*- coding: utf-8 -*-
import click


@click.command()
@click.pass_context
def watch_command(context):
    """
    Watch for change on your project Sass stylesheets then compile them to CSS.
    """
    click.echo("Not implemented yet!")



# -*- coding: utf-8 -*-
"""
Settings model
==============

"""
from boussole.conf import DEFAULT_SETTINGS


class Settings(object):
    """
    Settings model object

    Class init method fills object attributes from either given initial
    settings (if given) or default ones from ``DEFAULT_SETTINGS``.

    Keyword Arguments:
        initial (dict): A dictionnary of settings for initial values.
    """
    def __init__(self, initial=None):
        self._settings = {}
        self.update(initial or DEFAULT_SETTINGS)

    def clean(self, settings):
        """
        Filter given settings to keep only key names available in
        ``DEFAULT_SETTINGS``.

        Args:
            settings (dict): Loaded settings.

        Returns:
            dict: Settings object filtered.

        """
        return {k: v for k, v in settings.items() if k in DEFAULT_SETTINGS}

    def update(self, settings):
        """
        Update object attributes from given settings

        Args:
            settings (dict): Dictionnary of settings to update setting
                elements.

        Returns:
            dict: Dictionnary of all current saved settings.

        """
        settings = self.clean(settings)

        # Update internal dict
        self._settings.update(settings)

        # Push every setting items as class object attributes
        for k, v in settings.items():
            setattr(self, k, v)

        return self._settings



# -*- coding: utf-8 -*-
import os
import pytest

from boussole.conf.base_backend import SettingsBackendBase


def test_settings_base_open_ok_001(settings):
    """conf.base_backend.SettingsBackendBase: Open given filepath"""
    settings_loader = SettingsBackendBase()
    
    filepath = settings_loader.get_filepath(settings.fixtures_path)
    
    assert settings_loader.open(filepath) == """Fake settings file as SettingsBackendBase dont implement a full usable interface."""



# -*- coding: utf-8 -*-
import click

from boussole import __version__


@click.command()
@click.pass_context
def version_command(context):
    """
    Print out version information
    """
    click.echo("Boussole {}".format(__version__))



# -*- coding: utf-8 -*-
import os
import copy
import pytest

from boussole.conf.base_backend import SettingsBackendBase


def test_settings_base_clean_001(settings, sample_project_settings):
    """conf.base_backend.SettingsBackendBase: Ensure cleaning dont drop anything"""
    settings_loader = SettingsBackendBase()
    
    assert settings_loader.clean(sample_project_settings) == sample_project_settings



# -*- coding: utf-8 -*-
"""
Project configuration
=====================

Boussole work on per project configurations stored in a settings file for each
project.

Backends behavior is to search for a settings file in the given directory.
Almost all paths in settings will be expanded to absolute paths if they are
not allready so:

* If the path start with a home directory character, the home directory is used
  to expand the path;
* If the path is relative, expand it to absolute using directory from settings
  file location;

"""
import copy


# Settings manifest define default values and values patchs for each setting
SETTINGS_MANIFEST = {
    'COMPILER_ARGS': {
        'default': [],
        'patchs': [],
    },
    'LIBRARY_PATHS': {
        'default': [],
        'patchs': ['_patch_expand_paths'],
    },
    'SOURCES_PATHS': {
        'default': [],
        'patchs': ['_patch_expand_paths'],
    },
    'TARGET_PATH': {
        'default': None,
        'patchs': ['_patch_expand_path'],
    },
}


# Default values for initial settings object
DEFAULT_SETTINGS = {k: copy.deepcopy(v['default'])
                    for k, v in SETTINGS_MANIFEST.items()}



# -*- coding: utf-8 -*-
"""
Base settings backend
=====================

Backends are responsible to find settings file, parse it, load its values then
return a Settings object.

Each loaded setting values can be patched following settings manifest rules,
see :class:`boussole.conf.patcher`.

"""
import os

from boussole.exceptions import SettingsBackendError
from boussole.conf.model import Settings
from boussole.conf.patcher import SettingsPatcher


class SettingsBackendBase(SettingsPatcher):
    """
    Base project settings backend

    Args:
        basedir (str): Directory path where to search for settings file.
            Default is empty, meaning it will resolve path from current
            directory. You are advised to avoid this kind of usage and allways
            provide an absolute base directory.

    Attributes:
        _default_filename: Filename for settings file to load. Default to
            ``settings.txt`` but every backend should set their own filename.
    """
    _default_filename = 'settings.txt'

    def __init__(self, basedir=None):
        self.basedir = basedir or ''

    def get_filepath(self, path, filename=None):
        """
        Check and return the final filepath to settings

        Args:
            path (str): Directory path where to search for settings file.

        Keyword Arguments:
            filename (str): Filename to use to search for settings file. Will
                use ``_default_filename`` value if empty.

        Raises:
            boussole.exceptions.SettingsBackendError: If determined filepath
                does not exists or is a directory.

        Returns:
            string: Settings file path

        """
        filename = filename or self._default_filename
        settings_path = os.path.join(path, filename)

        if not os.path.exists(settings_path) or \
           not os.path.isfile(settings_path):
            msg = "Unable to load settings file: {}"
            raise SettingsBackendError(msg.format(settings_path))

        return settings_path

    def open(self, filepath):
        """
        Open settings backend to return its content

        Args:
            filepath (str): Settings object, depends from backend

        Returns:
            string: File content.

        """
        with open(filepath) as fp:
            content = fp.read()
        return content

    def parse(self, filepath, content):
        """
        Parse opened settings content

        Base method do nothing because parsing is dependent from backend.

        Args:
            filepath (str): Settings object, depends from backend
            content (str): Settings content from opened file, depends from
                backend.

        Returns:
            dict: Dictionnary containing parsed setting elements.

        """
        return {}

    def clean(self, settings):
        """
        Clean given settings for backend needs.

        Default backend only apply available patchs.

        Args:
            dict: Loaded settings.

        Returns:
            dict: Settings object cleaned.

        """
        return self.patch(settings)

    def load(self, filename=None):
        """
        Pretend to load the settings file from given path and optionnal
        filename.

        Final backend settings interface have to implement the final loading.

        Keyword Arguments:
            filename (str): Filename to use to search for settings file. Will
                use value from ``_default_filename`` class attribute if empty.

        Returns:
            boussole.conf.model.Settings: Settings object with loaded elements.

        """
        settings_path = self.get_filepath(self.basedir, filename)

        parsed = self.parse(settings_path, self.open(settings_path))

        settings = self.clean(parsed)

        return Settings(initial=settings)
